@@ -41,7 +41,7 @@ from swecli.ui.components.animations import Spinner, FlashingSymbol, ProgressInd
 from swecli.ui.components.task_progress import TaskProgressDisplay
 from swecli.ui.components.status_line import StatusLine
 from swecli.ui.autocomplete import SwecliCompleter
-from swecli.ui.formatters import OutputFormatter
+from swecli.ui.formatters_internal.output_formatter import OutputFormatter
 from swecli.ui.components.notifications import NotificationCenter
 from swecli.ui.formatters_internal.markdown_formatter import markdown_to_plain_text
 
@@ -150,7 +150,7 @@ class REPL:
         # UI Components
         self.spinner = Spinner(self.console)
         self.status_line = StatusLine(self.console)
-        self.output_formatter = OutputFormatter(self.console)
+        self.output_formatter = OutputFormatter(self.console, use_claude_style=True)
         self._notification_center = NotificationCenter(self.console)
 
         # UI state trackers
