@@ -10,7 +10,7 @@ from swecli.ui.autocomplete import (
     SlashCommand,
     SLASH_COMMANDS,
 )
-from swecli.ui.formatters import OutputFormatter
+from swecli.ui.formatters_internal.output_formatter import OutputFormatter
 
 __all__ = [
     "Spinner",
